@@ -2,6 +2,7 @@ package main
 
 import (
 	"bytes"
+	"context"
 	"encoding/json"
 	"fmt"
 	"io/ioutil"
@@ -9,6 +10,7 @@ import (
 	"strings"
 
 	"github.com/containers/buildah/manifests"
+	"github.com/containers/buildah/pkg/cli"
 	"github.com/containers/buildah/pkg/parse"
 	"github.com/containers/buildah/util"
 	"github.com/containers/common/pkg/auth"
@@ -16,6 +18,8 @@ import (
 	"github.com/containers/image/v5/manifest"
 	"github.com/containers/image/v5/transports"
 	"github.com/containers/image/v5/transports/alltransports"
+	"github.com/containers/image/v5/types"
+	"github.com/containers/storage"
 	digest "github.com/opencontainers/go-digest"
 	imgspecv1 "github.com/opencontainers/image-spec/specs-go/v1"
 	"github.com/pkg/errors"
@@ -24,14 +28,13 @@ import (
 )
 
 type manifestCreateOpts = struct {
-	osOverride, archOverride string
-	all                      bool
+	os, arch string
+	all      bool
 }
 type manifestAddOpts = struct {
-	osOverride, archOverride          string
-	os, arch, variant, osVersion      string
-	features, osFeatures, annotations []string
-	all                               bool
+	authfile, certDir, creds, os, arch, variant, osVersion string
+	features, osFeatures, annotations                      []string
+	tlsVerify, all                                         bool
 }
 type manifestRemoveOpts = struct{}
 type manifestAnnotateOpts = struct {
@@ -39,10 +42,6 @@ type manifestAnnotateOpts = struct {
 	features, osFeatures, annotations []string
 }
 type manifestInspectOpts = struct{}
-type manifestPushOpts = struct {
-	purge, quiet, all, tlsVerify, removeSignatures                        bool
-	authfile, certDir, creds, digestfile, format, signaturePolicy, signBy string
-}
 
 func init() {
 	var (
@@ -58,7 +57,7 @@ func init() {
 		manifestRemoveOpts          manifestRemoveOpts
 		manifestAnnotateOpts        manifestAnnotateOpts
 		manifestInspectOpts         manifestInspectOpts
-		manifestPushOpts            manifestPushOpts
+		manifestPushOpts            pushOptions
 	)
 	manifestCommand := &cobra.Command{
 		Use:   "manifest",
@@ -90,14 +89,15 @@ func init() {
 	manifestCreateCommand.SetUsageTemplate(UsageTemplate())
 	flags := manifestCreateCommand.Flags()
 	flags.BoolVar(&manifestCreateOpts.all, "all", false, "add all of the lists' images if the images to add are lists")
-	flags.StringVar(&manifestCreateOpts.osOverride, "override-os", "", "if any of the specified images is a list, choose the one for `os`")
-	if err := flags.MarkHidden("override-os"); err != nil {
-		panic(fmt.Sprintf("error marking override-os as hidden: %v", err))
+	flags.StringVar(&manifestCreateOpts.os, "os", "", "if any of the specified images is a list, choose the one for `os`")
+	if err := flags.MarkHidden("os"); err != nil {
+		panic(fmt.Sprintf("error marking --os as hidden: %v", err))
 	}
-	flags.StringVar(&manifestCreateOpts.archOverride, "override-arch", "", "if any of the specified images is a list, choose the one for `arch`")
-	if err := flags.MarkHidden("override-arch"); err != nil {
-		panic(fmt.Sprintf("error marking override-arch as hidden: %v", err))
+	flags.StringVar(&manifestCreateOpts.arch, "arch", "", "if any of the specified images is a list, choose the one for `arch`")
+	if err := flags.MarkHidden("arch"); err != nil {
+		panic(fmt.Sprintf("error marking --arch as hidden: %v", err))
 	}
+	flags.SetNormalizeFunc(cli.AliasFlags)
 	manifestCommand.AddCommand(manifestCreateCommand)
 
 	manifestAddCommand := &cobra.Command{
@@ -113,22 +113,19 @@ func init() {
 	}
 	manifestAddCommand.SetUsageTemplate(UsageTemplate())
 	flags = manifestAddCommand.Flags()
-	flags.StringVar(&manifestAddOpts.osOverride, "override-os", "", "if any of the specified images is a list, choose the one for `os`")
-	if err := flags.MarkHidden("override-os"); err != nil {
-		panic(fmt.Sprintf("error marking override-os as hidden: %v", err))
-	}
-	flags.StringVar(&manifestAddOpts.archOverride, "override-arch", "", "if any of the specified images is a list, choose the one for `arch`")
-	if err := flags.MarkHidden("override-arch"); err != nil {
-		panic(fmt.Sprintf("error marking override-arch as hidden: %v", err))
-	}
+	flags.StringVar(&manifestAddOpts.authfile, "authfile", auth.GetDefaultAuthFile(), "path of the authentication file. Use REGISTRY_AUTH_FILE environment variable to override")
+	flags.StringVar(&manifestAddOpts.certDir, "cert-dir", "", "use certificates at the specified path to access the registry")
+	flags.StringVar(&manifestAddOpts.creds, "creds", "", "use `[username[:password]]` for accessing the registry")
 	flags.StringVar(&manifestAddOpts.os, "os", "", "override the `OS` of the specified image")
 	flags.StringVar(&manifestAddOpts.arch, "arch", "", "override the `architecture` of the specified image")
-	flags.StringVar(&manifestAddOpts.variant, "variant", "", "override the `Variant` of the specified image")
+	flags.StringVar(&manifestAddOpts.variant, "variant", "", "override the `variant` of the specified image")
 	flags.StringVar(&manifestAddOpts.osVersion, "os-version", "", "override the OS `version` of the specified image")
 	flags.StringSliceVar(&manifestAddOpts.features, "features", nil, "override the `features` of the specified image")
 	flags.StringSliceVar(&manifestAddOpts.osFeatures, "os-features", nil, "override the OS `features` of the specified image")
 	flags.StringSliceVar(&manifestAddOpts.annotations, "annotation", nil, "set an `annotation` for the specified image")
+	flags.BoolVar(&manifestAddOpts.tlsVerify, "tls-verify", true, "require HTTPS and verify certificates when accessing the registry. TLS verification cannot be used when talking to an insecure registry.")
 	flags.BoolVar(&manifestAddOpts.all, "all", false, "add all of the list's images if the image is a list")
+	flags.SetNormalizeFunc(cli.AliasFlags)
 	manifestCommand.AddCommand(manifestAddCommand)
 
 	manifestRemoveCommand := &cobra.Command{
@@ -190,7 +187,7 @@ func init() {
 	}
 	manifestPushCommand.SetUsageTemplate(UsageTemplate())
 	flags = manifestPushCommand.Flags()
-	flags.BoolVar(&manifestPushOpts.purge, "purge", false, "remove the manifest list if push succeeds")
+	flags.BoolVar(&manifestPushOpts.rm, "rm", false, "remove the manifest list if push succeeds")
 	flags.BoolVar(&manifestPushOpts.all, "all", false, "also push the images in the list")
 	flags.StringVar(&manifestPushOpts.authfile, "authfile", auth.GetDefaultAuthFile(), "path of the authentication file. Use REGISTRY_AUTH_FILE environment variable to override")
 	flags.StringVar(&manifestPushOpts.certDir, "cert-dir", "", "use certificates at the specified path to access the registry")
@@ -203,8 +200,9 @@ func init() {
 	if err := flags.MarkHidden("signature-policy"); err != nil {
 		panic(fmt.Sprintf("error marking signature-policy as hidden: %v", err))
 	}
-	flags.BoolVar(&manifestPushOpts.tlsVerify, "tls-verify", true, "require HTTPS and verify certificates when accessing the registry")
+	flags.BoolVar(&manifestPushOpts.tlsVerify, "tls-verify", true, "require HTTPS and verify certificates when accessing the registry. TLS verification cannot be used when talking to an insecure registry.")
 	flags.BoolVarP(&manifestPushOpts.quiet, "quiet", "q", false, "don't output progress information when pushing lists")
+	flags.SetNormalizeFunc(cli.AliasFlags)
 	manifestCommand.AddCommand(manifestPushCommand)
 }
 
@@ -233,10 +231,13 @@ func manifestCreateCmd(c *cobra.Command, args []string, opts manifestCreateOpts)
 	}
 
 	for _, imageSpec := range imageSpecs {
-		ref, _, err := util.FindImage(store, "", systemContext, imageSpec)
+		ref, err := alltransports.ParseImageName(imageSpec)
 		if err != nil {
-			if ref, err = alltransports.ParseImageName(imageSpec); err != nil {
-				return err
+			if ref, err = alltransports.ParseImageName(util.DefaultTransport + imageSpec); err != nil {
+				// check if the local image exists
+				if ref, _, err = util.FindImage(store, "", systemContext, imageSpec); err != nil {
+					return err
+				}
 			}
 		}
 		if _, err = list.Add(getContext(), systemContext, ref, opts.all); err != nil {
@@ -252,6 +253,10 @@ func manifestCreateCmd(c *cobra.Command, args []string, opts manifestCreateOpts)
 }
 
 func manifestAddCmd(c *cobra.Command, args []string, opts manifestAddOpts) error {
+	if err := auth.CheckAuthFile(opts.authfile); err != nil {
+		return err
+	}
+
 	listImageSpec := ""
 	imageSpec := ""
 	switch len(args) {
@@ -285,10 +290,13 @@ func manifestAddCmd(c *cobra.Command, args []string, opts manifestAddOpts) error
 		return err
 	}
 
-	ref, _, err := util.FindImage(store, "", systemContext, imageSpec)
+	ref, err := alltransports.ParseImageName(imageSpec)
 	if err != nil {
-		if ref, err = alltransports.ParseImageName(imageSpec); err != nil {
-			return err
+		if ref, err = alltransports.ParseImageName(util.DefaultTransport + imageSpec); err != nil {
+			// check if the local image exists
+			if ref, _, err = util.FindImage(store, "", systemContext, imageSpec); err != nil {
+				return err
+			}
 		}
 	}
 
@@ -297,11 +305,17 @@ func manifestAddCmd(c *cobra.Command, args []string, opts manifestAddOpts) error
 		return err
 	}
 
-	ctx := getContext()
-
-	digest, err := list.Add(ctx, systemContext, ref, opts.all)
+	digest, err := list.Add(getContext(), systemContext, ref, opts.all)
 	if err != nil {
-		return err
+		var storeErr error
+		// check if the local image exists
+		if ref, _, storeErr = util.FindImage(store, "", systemContext, imageSpec); storeErr != nil {
+			return err
+		}
+		digest, storeErr = list.Add(getContext(), systemContext, ref, opts.all)
+		if storeErr != nil {
+			return err
+		}
 	}
 
 	if opts.os != "" {
@@ -457,10 +471,9 @@ func manifestAnnotateCmd(c *cobra.Command, args []string, opts manifestAnnotateO
 		return err
 	}
 
-	ctx := getContext()
-
 	digest, err := digest.Parse(imageSpec)
 	if err != nil {
+		ctx := getContext()
 		ref, _, err := util.FindImage(store, "", systemContext, imageSpec)
 		if err != nil {
 			return err
@@ -556,9 +569,13 @@ func manifestInspectCmd(c *cobra.Command, args []string, opts manifestInspectOpt
 		return errors.Wrapf(err, "error building system context")
 	}
 
+	return manifestInspect(getContext(), store, systemContext, imageSpec)
+}
+
+func manifestInspect(ctx context.Context, store storage.Store, systemContext *types.SystemContext, imageSpec string) error {
 	refs, err := util.ResolveNameToReferences(store, systemContext, imageSpec)
 	if err != nil {
-		return errors.Wrapf(err, "error resolving image names")
+		logrus.Debugf("error parsing reference to image %q: %v", imageSpec, err)
 	}
 
 	if ref, _, err := util.FindImage(store, "", systemContext, imageSpec); err == nil {
@@ -566,8 +583,10 @@ func manifestInspectCmd(c *cobra.Command, args []string, opts manifestInspectOpt
 	} else if ref, err := alltransports.ParseImageName(imageSpec); err == nil {
 		refs = append(refs, ref)
 	}
+	if len(refs) == 0 {
+		return errors.Errorf("error locating images with names %v", imageSpec)
+	}
 
-	ctx := getContext()
 	var (
 		latestErr error
 		result    []byte
@@ -619,7 +638,7 @@ func manifestInspectCmd(c *cobra.Command, args []string, opts manifestInspectOpt
 	return nil
 }
 
-func manifestPushCmd(c *cobra.Command, args []string, opts manifestPushOpts) error {
+func manifestPushCmd(c *cobra.Command, args []string, opts pushOptions) error {
 	if err := auth.CheckAuthFile(opts.authfile); err != nil {
 		return err
 	}
@@ -648,12 +667,15 @@ func manifestPushCmd(c *cobra.Command, args []string, opts manifestPushOpts) err
 	if err != nil {
 		return err
 	}
-
 	systemContext, err := parse.SystemContextFromOptions(c)
 	if err != nil {
 		return errors.Wrapf(err, "error building system context")
 	}
 
+	return manifestPush(systemContext, store, listImageSpec, destSpec, opts)
+}
+
+func manifestPush(systemContext *types.SystemContext, store storage.Store, listImageSpec, destSpec string, opts pushOptions) error {
 	_, listImage, err := util.FindImage(store, "", systemContext, listImageSpec)
 	if err != nil {
 		return err
@@ -664,8 +686,6 @@ func manifestPushCmd(c *cobra.Command, args []string, opts manifestPushOpts) err
 		return err
 	}
 
-	ctx := getContext()
-
 	dest, err := alltransports.ParseImageName(destSpec)
 	if err != nil {
 		return err
@@ -699,9 +719,9 @@ func manifestPushCmd(c *cobra.Command, args []string, opts manifestPushOpts) err
 		options.ReportWriter = os.Stderr
 	}
 
-	_, digest, err := list.Push(ctx, dest, options)
+	_, digest, err := list.Push(getContext(), dest, options)
 
-	if err == nil && opts.purge {
+	if err == nil && opts.rm {
 		_, err = store.DeleteImage(listImage.ID, true)
 	}
 
